@@ -1,3 +1,9 @@
+/*
+* (C) 2009 Jack Lloyd
+*
+* Distributed under the terms of the Botan license
+*/
+
 #include <vector>
 #include <string>
 #include <fstream>
@@ -5,11 +11,12 @@
 #include <cstdlib>
 
 #include <botan/bigint.h>
+#include <botan/exceptn.h>
 #include <botan/numthry.h>
-#include <botan/rng.h>
 using namespace Botan;
 
 #include "common.h"
+#include "validate.h"
 
 #define DEBUG 0
 
@@ -18,22 +25,22 @@ u32bit check_sub(const std::vector<std::string>&);
 u32bit check_mul(const std::vector<std::string>&);
 u32bit check_sqr(const std::vector<std::string>&);
 u32bit check_div(const std::vector<std::string>&);
-u32bit check_mod(const std::vector<std::string>&);
+u32bit check_mod(const std::vector<std::string>&,
+                 Botan::RandomNumberGenerator& rng);
 u32bit check_shr(const std::vector<std::string>&);
 u32bit check_shl(const std::vector<std::string>&);
 
 u32bit check_powmod(const std::vector<std::string>&);
-u32bit check_primetest(const std::vector<std::string>&);
+u32bit check_primetest(const std::vector<std::string>&,
+                       Botan::RandomNumberGenerator&);
 
-u32bit do_bigint_tests(const std::string& filename)
+u32bit do_bigint_tests(const std::string& filename,
+                       Botan::RandomNumberGenerator& rng)
    {
    std::ifstream test_data(filename.c_str());
 
    if(!test_data)
-       {
-       std::cout << "Couldn't open test file " << filename << std::endl;
-       std::exit(1);
-       }
+      throw Botan::Stream_IO_Error("Couldn't open test file " + filename);
 
    u32bit errors = 0, alg_count = 0;
    std::string algorithm;
@@ -43,10 +50,8 @@ u32bit do_bigint_tests(const std::string& filename)
    while(!test_data.eof())
       {
       if(test_data.bad() || test_data.fail())
-         {
-         std::cout << "File I/O error." << std::endl;
-         std::exit(1);
-         }
+         throw Botan::Stream_IO_Error("File I/O error reading from " +
+                                      filename);
 
       std::string line;
       std::getline(test_data, line);
@@ -97,7 +102,7 @@ u32bit do_bigint_tests(const std::string& filename)
       else if(algorithm.find("Division") != std::string::npos)
          new_errors = check_div(substr);
       else if(algorithm.find("Modulo") != std::string::npos)
-         new_errors = check_mod(substr);
+         new_errors = check_mod(substr, rng);
       else if(algorithm.find("LeftShift") != std::string::npos)
          new_errors = check_shl(substr);
       else if(algorithm.find("RightShift") != std::string::npos)
@@ -105,7 +110,7 @@ u32bit do_bigint_tests(const std::string& filename)
       else if(algorithm.find("ModExp") != std::string::npos)
          new_errors = check_powmod(substr);
       else if(algorithm.find("PrimeTest") != std::string::npos)
-         new_errors = check_primetest(substr);
+         new_errors = check_primetest(substr, rng);
       else
          std::cout << "Unknown MPI test " << algorithm << std::endl;
 
@@ -126,6 +131,8 @@ u32bit do_bigint_tests(const std::string& filename)
    return errors;
    }
 
+namespace {
+
 // c==expected, d==a op b, e==a op= b
 u32bit results(std::string op,
                const BigInt& a, const BigInt& b,
@@ -158,6 +165,8 @@ u32bit results(std::string op,
       }
    }
 
+}
+
 u32bit check_add(const std::vector<std::string>& args)
    {
    BigInt a(args[0]);
@@ -249,7 +258,8 @@ u32bit check_div(const std::vector<std::string>& args)
    return results("/", a, b, c, d, e);
    }
 
-u32bit check_mod(const std::vector<std::string>& args)
+u32bit check_mod(const std::vector<std::string>& args,
+                 Botan::RandomNumberGenerator& rng)
    {
    BigInt a(args[0]);
    BigInt b(args[1]);
@@ -268,7 +278,7 @@ u32bit check_mod(const std::vector<std::string>& args)
    /* Won't work for us, just pick one at random */
    while(b_word == 0)
       for(u32bit j = 0; j != 2*sizeof(word); j++)
-         b_word = (b_word << 4) ^ Global_RNG::random();
+         b_word = (b_word << 4) ^ rng.next_byte();
 
    b = b_word;
 
@@ -331,12 +341,13 @@ u32bit check_powmod(const std::vector<std::string>& args)
    }
 
 /* Make sure that n is prime or not prime, according to should_be_prime */
-u32bit check_primetest(const std::vector<std::string>& args)
+u32bit check_primetest(const std::vector<std::string>& args,
+                       Botan::RandomNumberGenerator& rng)
    {
    BigInt n(args[0]);
    bool should_be_prime = (args[1] == "1");
 
-   bool is_prime = Botan::verify_prime(n);
+   bool is_prime = Botan::verify_prime(n, rng);
 
    if(is_prime != should_be_prime)
       {
